package cc.winboll.studio.miniplayer;

/**
 * @Author ZhanGSKen<zhangsken@188.com>
 * @Date 2025/06/29 10:10
 * @Describe Midi 播放窗口
 */
import android.app.Activity;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.SeekBar;
import android.widget.TextView;
import com.hjq.toast.ToastUtils;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class MidiPlayerActivity extends WinBoLLActivity {

    public static final String TAG = "MidiPlayerActivity";

    private MidiPlayer mMidiPlayer;
    private Button mPlayBtn, mPauseBtn, mStopBtn;
    private ListView mTrackListView;
    private ListView mFileListView; // 文件列表
    private TrackAdapter mTrackAdapter;
    private TextView mFileNameTv;
    private List<File> mMidiFileList = new ArrayList<>(); // 存储midi文件列表

    @Override
    public Activity getActivity() {
        return this;
    }

    @Override
    public String getTag() {
        return TAG;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_midi_player); // 需确保布局文件存在

        // 初始化控件
        mPlayBtn = (Button) findViewById(R.id.btn_play);
        mPauseBtn = (Button) findViewById(R.id.btn_pause);
        mStopBtn = (Button) findViewById(R.id.btn_stop);
        mTrackListView = (ListView) findViewById(R.id.lv_tracks);
        mFileListView = (ListView) findViewById(R.id.lv_midi_files);
        mFileNameTv = (TextView) findViewById(R.id.tv_file_name);

        // 检查系统版本兼容性
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            initMidiPlayer();
            loadMidiFileList(); // 加载本地MIDI文件
            initControlButtons();
        } else {
            mFileNameTv.setText("当前设备不支持MIDI播放（需Android 6.0+）");
            disableButtons();
        }
    }

    // 初始化MIDI播放器并设置连接回调
    private void initMidiPlayer() {
        mMidiPlayer = new MidiPlayer(this);
        // 设置合成器连接状态回调（Java 7使用匿名内部类替代Lambda）
        mMidiPlayer.setOnSynthConnectedListener(new MidiPlayer.OnSynthConnectedListener() {
				@Override
				public void onConnected(final boolean success) {
					runOnUiThread(new Runnable() {
							@Override
							public void run() {
								if (success) {
									ToastUtils.show("MIDI合成器连接成功");
								} else {
									ToastUtils.show("未找到MIDI合成器，请安装第三方合成器应用");
								}
							}
						});
				}
			});
    }

    // 加载本地MIDI文件列表
    private void loadMidiFileList() {
        File midiDir = new File(getFilesDir(), "midi");
        // 检查目录是否存在
        if (!midiDir.exists()) {
            midiDir.mkdirs(); // 创建目录
            mFileNameTv.setText("midi目录已创建，请放入文件");
            return;
        }

        // 筛选.mid和.midi文件
        File[] files = midiDir.listFiles();
        if (files != null) {
            for (File file : files) {
                String name = file.getName().toLowerCase();
                if (name.endsWith(".mid") || name.endsWith(".midi")) {
                    mMidiFileList.add(file);
                }
            }
        }

        // 显示文件列表
        if (mMidiFileList.isEmpty()) {
            mFileNameTv.setText("midi目录中无可用文件");
        } else {
            showFileList();
        }
    }

    // 显示文件列表到ListView
    private void showFileList() {
        List<String> fileNameList = new ArrayList<>();
        for (File file : mMidiFileList) {
            fileNameList.add(file.getName());
        }

        // 设置文件列表适配器
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(
            this,
            android.R.layout.simple_list_item_1,
            fileNameList
        );
        mFileListView.setAdapter(adapter);

        // 文件点击事件：加载选中的MIDI文件（Java 7使用匿名内部类替代Lambda）
        mFileListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
				@Override
				public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
					mMidiPlayer.stop(); // 停止当前播放
					File selectedFile = mMidiFileList.get(position);
					boolean loaded = mMidiPlayer.loadMidiFile(selectedFile);
					if (loaded) {
						mFileNameTv.setText("当前文件：" + selectedFile.getName());
						initTrackList(); // 刷新轨道列表
						ToastUtils.show("已加载：" + selectedFile.getName());
					} else {
						mFileNameTv.setText("文件加载失败：" + selectedFile.getName());
					}
				}
			});
    }

    // 初始化轨道列表（显示所有轨道的控制项）
    private void initTrackList() {
        mTrackAdapter = new TrackAdapter(this, mMidiPlayer);
        mTrackListView.setAdapter(mTrackAdapter);
    }

    // 初始化播放控制按钮
    private void initControlButtons() {
        mPlayBtn.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					if (mMidiPlayer != null) {
						mMidiPlayer.start();
					}
				}
			});

        mPauseBtn.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					if (mMidiPlayer != null) {
						mMidiPlayer.pause();
					}
				}
			});

        mStopBtn.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					if (mMidiPlayer != null) {
						mMidiPlayer.stop();
					}
				}
			});
    }

    // 禁用所有控制按钮
    private void disableButtons() {
        mPlayBtn.setEnabled(false);
        mPauseBtn.setEnabled(false);
        mStopBtn.setEnabled(false);
    }

    // 轨道控制适配器（每个轨道显示静音按钮和音量条）
    private class TrackAdapter extends android.widget.BaseAdapter {
        private Context mContext;
        private MidiPlayer mPlayer;

        public TrackAdapter(Context context, MidiPlayer player) {
            mContext = context;
            mPlayer = player;
        }

        @Override
        public int getCount() {
            return mPlayer != null ? mPlayer.getTrackCount() : 0;
        }

        @Override
        public Object getItem(int position) {
            return null;
        }

        @Override
        public long getItemId(int position) {
            return position;
        }

        @Override
        public View getView(final int position, View convertView, ViewGroup parent) {
            // 加载轨道项布局（需创建item_track.xml）
            View view = getLayoutInflater().inflate(R.layout.item_track, parent, false);
            TextView trackNameTv = (TextView) view.findViewById(R.id.tv_track_name);
            final Button muteBtn = (Button) view.findViewById(R.id.btn_mute);
            SeekBar volumeSb = (SeekBar) view.findViewById(R.id.sb_volume);

            // 设置轨道名称
            trackNameTv.setText("轨道 " + (position + 1));

            // 静音按钮逻辑
            muteBtn.setOnClickListener(new View.OnClickListener() {
					@Override
					public void onClick(View v) {
						boolean isMuted = muteBtn.isSelected();
						mPlayer.setTrackMute(position, !isMuted);
						muteBtn.setSelected(!isMuted);
						muteBtn.setText(isMuted ? "静音" : "已静音");
					}
				});

            // 音量条（示例：实际需结合MIDI事件处理）
            volumeSb.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
					@Override
					public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
						// 可添加音量控制逻辑（如修改MIDI音量事件）
					}

					@Override
					public void onStartTrackingTouch(SeekBar seekBar) {}

					@Override
					public void onStopTrackingTouch(SeekBar seekBar) {}
				});

            return view;
        }
    }

    // 日志查看按钮点击事件
    public void onLog(View view) {
        App.getWinBoLLActivityManager().startLogActivity(this);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        // 释放资源
        if (mMidiPlayer != null) {
            mMidiPlayer.stop();
        }
    }
}

